/*  xpsk31.h
 *
 *  Header file for xpsk31
 */

/*
 *  xpsk31: An application to transmit and receive
 *  PSK31 signals using a computer's sound card
 *
 *  Copyright (C) 2001  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */


#ifndef XPSK31_H
#define XPSK31_H  1

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <time.h>
#include <math.h>

/* General definitions for signal processing */
#define BAUD_RATE			31.25	/* PSK baude rate */
#define DSP_SPEED			48000	/* DSP sampling speed in samples/sec */

/* FFT (waterfall) function parameters */
#define FFT_SIZE			256		/* Size of fft input arrays */
#define FFT_SIZE2			128		/* Size/2 of fft input arrays */
#define WFALL_WIDTH			128		/* Num. of waterfall display columns */
#define BINMAX_AVE_WIN		5		/* Length of bim max averaging window*/

/* Phase detector parameter definitions */
#define AUDIO_FREQUENCY		500		/* Frequency in Hz of phase detector */
#define AUDIO_FREQx10		5000	/* As above but * 10, needed in calc */
#define PHASE_AVE_WINDOW	5		/* Length of phase averager window   */
#define SQUELCH_WINDOW		10		/* Length of squelch averager window */
#define PHASE_SHIFT_0		0		/* PSK phase reference    */
#define PHASE_SHIFT_90		1		/* PSK 90 phase advance   */
#define PHASE_SHIFT_180		2		/* PSK 180 phase reversal */
#define PHASE_SHIFT_270		3		/* PSK 90 phase retard    */

/* Control flags */
#define TRANSMIT_TAG		0x000001 /* Send tag string  */
#define SQUELCH_OPEN		0x000002 /* Squelch is open  */
#define MODE_BPSK			0x000004 /* Mode is BPSK     */
#define MODE_QPSK_USB		0x000008 /* Mode is QPSK-USB */
#define MODE_QPSK_LSB		0x000010 /* Mode is QPSK-LSB */
#define ENABLE_CAT			0x000020 /* Enable Trcvr CAT */
#define ENABLE_CAT_847		0x000040 /* Enable Trcvr CAT */
#define ENABLE_CAT_857		0x000080 /* Enable Trcvr CAT */
#define ENABLE_RTS_CTL		0x000100 /* Enable RTS control of PTT */
#define RECORD_QSO			0x000200 /* Enable recording of QSO's   */
#define RECEIVE_MODE		0x000400 /* xpsk31 is in Receive mode   */
#define TRANSMIT_MODE		0x000800 /* xpsk31 is in Transmit mode  */
#define TRANSMIT_MACRO		0x001000 /* Start transmission of Macro */
#define TRANSMIT_KEYBD		0x002000 /* Start transmission of Macro */
#define CAPITAL_LETTERS		0x004000 /* Capitalize letters in Trans */
#define CHANNEL_A_SELECT	0x008000 /* Detector channel A selected */
#define SOUND_CARD_SETUP	0x010000 /* Sound card has been set-up  */
#define SERIAL_IO_ERROR		0x020000 /* A serial i/o error occured  */
#define DSP_IO_ERROR		0x040000 /* A DSP read/write error      */
#define SAVE_RECORD			0x080000 /* Record was saved to file    */
#define ENABLE_AFC			0x100000 /* Enable Auto Freq. Control   */
#define ENABLE_MAGNIPHASE	0x200000 /* Enable display of magniphase   */
#define REVERSE_VIDEO		0x400000 /* Enable reversal of wfall color */

/* Receive magniphase scale factor */
#define MAGNIPHASE_SCALE	44

/* Some character definitions */
#define HT				0x09 /* Horizontal Tab  */
#define CR				0x0d /* Carriage Return */
#define LF				0x0a /* Line Feed       */
#define BS				0x08 /* Back Space      */
#define NO_CHARACTER	0x80 /* No psk31 character decoded yet */

/* Number and size of labels for commands  */
#define NUM_OF_LABELS	9
#define LABEL_LENGTH	20

/* Return values */
#define SUCCESS		0
#define ERROR		1

/* Polynomials for the convolution encoder */
#define POLYNOM1	0x19
#define POLYNOM2	0x17

/* Table of tag strings */
#define TAG_TABLE \
{ \
  "own-call", "own-name" , "own-qth" , \
  "own-loc" , "own-rst"  , "rem-call", \
  "rem-name", "rem-qth"  , "rem-loc" , \
  "rem-rst" , "date-time", "op-freq" , \
  "app-version" \
}

/* Number of recognizable tags */
#define NUM_OF_TAGS		13

/* Receive and Transmit status indicators */
#define RECV_OFF      "<span  background=\"grey\" > RECEIVE (OFF) </span>"
#define RECV_BPSK     "<span background=\"green\" > RECEIVE (BPSK) </span>"
#define RECV_QPSK_USB "<span background=\"green\" > RECEIVE (QPSK-USB) </span>"
#define RECV_QPSK_LSB "<span background=\"green\" > RECEIVE (QPSK-LSB) </span>"
#define XMIT_OFF      "<span background=\"grey\" > TRANSMIT (OFF) </span>"
#define XMIT_BPSK     "<span  background=\"red\" > TRANSMIT (BPSK) </span>"
#define XMIT_QPSK_USB "<span  background=\"red\" > TRANSMIT (QPSK-USB) </span>"
#define XMIT_QPSK_LSB "<span  background=\"red\" > TRANSMIT (QPSK-LSB) </span>"

/*-------------------------------------------------------------------*/

/* Trellis data structure for the Viterbi decoder */
typedef struct
{
  int      distance; /* Total excess distance to this state  */
  long int estimate; /* Estimate of transmitted bit sequence */
} trellis_state_t;

/* Configuration parameters structure */
typedef struct
{
  char
	mixer[15],   /* Mixer device     */
	dsp_in[15],  /* Input DSP device */
	dsp_out[15], /* Outpt DSP device */
	rec_src[10], /* Capture source   */
	lev_dev[10], /* Rec level device */
	mode[7],     /* Mono | Stereo    */
	channel[6],  /* Left | Right     */
	serial1[15], /* CAT serial port 1*/
	serial2[15], /* CAT serial port 2*/
	serial3[15]; /* RTS serial port  */

  int
	rec_lev,    /* Capture/record level */
	vol_lev,    /* Output volume level  */
	sqlch_thr,  /* Squelch threshold    */
	tone_dur,   /* Preamble tone duration   */
	num_rev,    /* Pre/post-amble reversals */
	wpm,        /* Morse encoder speed: wpm */
	wwrap,      /* Word wrap column number  */
	psk_elem2,	/* Duration of half PSK element in dsp samples */
	psk_elem,   /* Duration of a PSK element in dsp samples */
	tone_period,/* Period of audio tone in dsp cycles */
	fft_stride; /* FFT stride over input data (dsp samples) */

  FILE
	*qso_record_fp,  /* File for recording QSO's */
	*log_adif_fp,    /* File for ADIF QSO record */
	*station_log_fp; /* File for station's log   */

  /* Local operator data */
  char
	call[15],
	name[13],
	zone[15],
	qth[13],
	loc[7];

  /* Tx/Rx power, N.Fig, antennas */
  char
	tx[12],
	tx_power[10],
	tx_ant[16],
	rx[12],
	rx_nfig[8],
	rx_ant[16];

} rc_data_t;

/* QSO record type definition */
typedef struct
{
  /* QSO data */
  char
	dx_call[15],  /* Remote (dx) station callsign */
	dx_rst[4],    /* My RST report to Dx station  */
	my_rst[4],    /* My RST report to Dx station  */
	dx_name[13],  /* Remote station operator name */
	dx_qth[13],   /* Remote station QTH loc. name */
	dx_loc[7],    /* Remote station QTH locator   */
	date[12],     /* Normal format QSO date  */
	date_adif[9], /* QSO date in ADIF format */
	time[6],      /* Normal format QSO time  */
	time_adif[5], /* QSO time in ADIF format */
	freq[14],     /* QSO operating frequency MHz */
	mode[12],     /* Mode of operation */
	via[13];      /* Via which path or relay Sat */

  /* Remarks on QSO */
  char remarks[46];

} qso_record_t;

/*-------------------------------------------------------------------*/

/*** Tables for various trig functions ***/

/* Arc tan correction table */
#define ARC_TAN_TABLE \
{ \
   0,  1,  3,  4,  5,  6,  8,  9, 10, 11, 13, 14, \
  15, 16, 17, 18, 20, 21, 22, 23, 24, 25, 26, 27, \
  28, 29, 30, 31, 32, 33, 34, 35, 35, 36, 37, 38, \
  39, 39, 40, 41, 42, 42, 43, 44, 44, 45          \
}

/*-------------------------------------------------------------------*/

/*
   The VARICODE Alphabet

   The codes are transmitted left bit first, with '0' representing
   a phase reversal on BPSK and '1' representing a steady carrier.
   A minimum of two zeros is inserted between characters.
   Some implementations may not handle all the codes below 32.
 */

#define NUM_VARICODE_CHARS  128

/* 'NUL' character used as 'last character' flag */
#define NULL_CHAR	0x00

#define VARICODE_ALPHABET \
{ \
  /* ASCII   VARICODE       HEX */ \
  /* NUL   10 1010 1011 */  0x2ab, \
  /* SOH   10 1101 1011 */  0x2db, \
  /* STX   10 1110 1101 */  0x2ed, \
  /* ETX   11 0111 0111 */  0x377, \
  /* EOT   10 1110 1011 */  0x2eb, \
  /* ENQ   11 0101 1111 */  0x35f, \
  /* ACK   10 1110 1111 */  0x2ef, \
  /* BEL   10 1111 1101 */  0x2fd, \
  /* BS    10 1111 1111 */  0x2ff, \
  /* TAB      1110 1111 */  0x0ef, \
  /* LF          1 1101 */  0x01d, \
  /* VT    11 0110 1111 */  0x36f, \
  /* FF    10 1101 1101 */  0x2dd, \
  /* CR          1 1111 */  0x01f, \
  /* SO    11 0111 0101 */  0x375, \
  /* SI    11 1010 1011 */  0x3ab, \
  /* DLE   10 1111 0111 */  0x2f7, \
  /* DC1   10 1111 0101 */  0x2f5, \
  /* DC2   11 1010 1101 */  0x3ad, \
  /* DC3   11 1010 1111 */  0x3af, \
  /* DC4   11 0101 1011 */  0x35b, \
  /* NAK   11 0110 1011 */  0x36b, \
  /* SYN   11 0110 1101 */  0x36d, \
  /* ETB   11 0101 0111 */  0x357, \
  /* CAN   11 0111 1011 */  0x37b, \
  /* EM    11 0111 1101 */  0x37d, \
  /* SUB   11 1011 0111 */  0x3b7, \
  /* ESC   11 0101 0101 */  0x355, \
  /* FS    11 0101 1101 */  0x35d, \
  /* GS    11 1011 1011 */  0x3bb, \
  /* RS    10 1111 1011 */  0x2fb, \
  /* US    11 0111 1111 */  0x37f, \
  /*                  1 */  0x001, \
  /* !      1 1111 1111 */  0x1ff, \
  /* "      1 0101 1111 */  0x15f, \
  /* #      1 1111 0101 */  0x1f5, \
  /* $      1 1101 1011 */  0x1db, \
  /* %     10 1101 0101 */  0x2d5, \
  /* &     10 1011 1011 */  0x2bb, \
  /* '      1 0111 1111 */  0x17f, \
  /* (        1111 1011 */  0x0fb, \
  /* )        1111 0111 */  0x0f7, \
  /* *      1 0110 1111 */  0x16f, \
  /* +      1 1101 1111 */  0x1df, \
  /* ,         111 0101 */  0x075, \
  /* -          11 0101 */  0x035, \
  /* .         101 0111 */  0x057, \
  /* /      1 1010 1111 */  0x1af, \
  /* 0        1011 0111 */  0x0b7, \
  /* 1        1011 1101 */  0x0bd, \
  /* 2        1110 1101 */  0x0ed, \
  /* 3        1111 1111 */  0x0ff, \
  /* 4      1 0111 0111 */  0x177, \
  /* 5      1 0101 1011 */  0x15b, \
  /* 6      1 0110 1011 */  0x16b, \
  /* 7      1 1010 1101 */  0x1ad, \
  /* 8      1 1010 1011 */  0x1ab, \
  /* 9      1 1011 0111 */  0x1b7, \
  /* :        1111 0101 */  0x0f5, \
  /* ;      1 1011 1101 */  0x1bd, \
  /* <      1 1110 1101 */  0x1ed, \
  /* =         101 0101 */  0x055, \
  /* >      1 1101 0111 */  0x1d7, \
  /* ?     10 1010 1111 */  0x2af, \
  /* @     10 1011 1101 */  0x2bd, \
  /* A         111 1101 */  0x07d, \
  /* B        1110 1011 */  0x0eb, \
  /* C        1010 1101 */  0x0ad, \
  /* D        1011 0101 */  0x0b5, \
  /* E         111 0111 */  0x077, \
  /* F        1101 1011 */  0x0db, \
  /* G        1111 1101 */  0x0fd, \
  /* H      1 0101 0101 */  0x155, \
  /* I         111 1111 */  0x07f, \
  /* J      1 1111 1101 */  0x1fd, \
  /* K      1 0111 1101 */  0x17d, \
  /* L        1101 0111 */  0x0d7, \
  /* M        1011 1011 */  0x0bb, \
  /* N        1101 1101 */  0x0dd, \
  /* O        1010 1011 */  0x0ab, \
  /* P        1101 0101 */  0x0d5, \
  /* Q      1 1101 1101 */  0x1dd, \
  /* R        1010 1111 */  0x0af, \
  /* S         110 1111 */  0x06f, \
  /* T         110 1101 */  0x06d, \
  /* U      1 0101 0111 */  0x157, \
  /* V      1 1011 0101 */  0x1b5, \
  /* W      1 0101 1101 */  0x15d, \
  /* X      1 0111 0101 */  0x175, \
  /* Y      1 0111 1011 */  0x17b, \
  /* Z     10 1010 1101 */  0x2ad, \
  /* [      1 1111 0111 */  0x1f7, \
  /* \      1 1110 1111 */  0x1ef, \
  /* ]      1 1111 1011 */  0x1fb, \
  /* ^     10 1011 1111 */  0x2bf, \
  /* _      1 0110 1101 */  0x16d, \
  /* `     10 1101 1111 */  0x2df, \
  /* a             1011 */  0x00b, \
  /* b         101 1111 */  0x05f, \
  /* c          10 1111 */  0x02f, \
  /* d          10 1101 */  0x02d, \
  /* e               11 */  0x003, \
  /* f          11 1101 */  0x03d, \
  /* g         101 1011 */  0x05b, \
  /* h          10 1011 */  0x02b, \
  /* i             1101 */  0x00d, \
  /* j      1 1110 1011 */  0x1eb, \
  /* k        1011 1111 */  0x0bf, \
  /* l           1 1011 */  0x01b, \
  /* m          11 1011 */  0x03b, \
  /* n             1111 */  0x00f, \
  /* o              111 */  0x007, \
  /* p          11 1111 */  0x03f, \
  /* q      1 1011 1111 */  0x1bf, \
  /* r           1 0101 */  0x015, \
  /* s           1 0111 */  0x017, \
  /* t              101 */  0x005, \
  /* u          11 0111 */  0x037, \
  /* v         111 1011 */  0x07b, \
  /* w         110 1011 */  0x06b, \
  /* x        1101 1111 */  0x0df, \
  /* y         101 1101 */  0x05d, \
  /* z      1 1101 0101 */  0x1d5, \
  /* {     10 1011 0111 */  0x2b7, \
  /* |      1 1011 1011 */  0x1bb, \
  /* }     10 1011 0101 */  0x2b5, \
  /* ~     10 1101 0111 */  0x2d7, \
  /* DEL   11 1011 0101 */  0x3b5  \
}

/*
   The CONVOLUTIONAL Code

   The left column contains the 32 combinations of a run of five
   Varicode bits, transmitted left bit first. The right column is
   the corresponding phase shift to be applied to the carrier, with
   0 meaning no shift, 1 meaning advance by 90, 2 meaning polarity
   reversal and 3 meaning retard by 90.  A continuous phase advance
   is the same as an HF frequency shift.
 */

#define CONVOLUTIONAL_CODE \
{ \
  /* Bits      Phase */ \
  /* 00000 */  2,       \
  /* 00001 */  1,       \
  /* 00010 */  3,       \
  /* 00011 */  0,       \
  /* 00100 */  3,       \
  /* 00101 */  0,       \
  /* 00110 */  2,       \
  /* 00111 */  1,       \
  /* 01000 */  0,       \
  /* 01001 */  3,       \
  /* 01010 */  1,       \
  /* 01011 */  2,       \
  /* 01100 */  1,       \
  /* 01101 */  2,       \
  /* 01110 */  0,       \
  /* 01111 */  3,       \
  /* 10000 */  1,       \
  /* 10001 */  2,       \
  /* 10010 */  0,       \
  /* 10011 */  3,       \
  /* 10100 */  0,       \
  /* 10101 */  3,       \
  /* 10110 */  1,       \
  /* 10111 */  2,       \
  /* 11000 */  3,       \
  /* 11001 */  0,       \
  /* 11010 */  2,       \
  /* 11011 */  1,       \
  /* 11100 */  2,       \
  /* 11101 */  1,       \
  /* 11110 */  3,       \
  /* 11111 */  0        \
}

/*
   For example, the "space" symbol - a single 1 preceded and followed
   by zeros - would be represented by successive run-of-five groups
   00000, 00001, 00010, 00100, 01000, 10000, 00000.
   The transmitter will send the QPSK pattern .. 2,1,3,3,0,1,2, ..
   Note that a continuous sequence of zeros (the idle sequence)
   gives continuous reversals, the same as BPSK.
 */

/*-------------------------------------------------------------------*/

/* Definitions for ascii to Morse code conversion */
#define NUM_MORSE_CHARS           55 /* Number of chars in Morse table */

/* ASCII equivalents to Morse hex code. Last */
/* one (*) used for unrecognized characters. */
#define ASCII_CHAR \
{ \
  'A','B','C','D','E','F','G','H','I','J','K','L','M',  \
  'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',  \
  '1','2','3','4','5','6','7','8','9','0','.',',',':',  \
  '?','\'','-','/','(','"',';','$','#','<','!','@',']', \
  '=','~',' '										    \
}

/* Hex equivalents to Morse code chars above. Formed   */
/* by starting with a 1 and following with a 0 for     */
/* dash and 1 for dit e.g: A = ditdah = 101 = Hex 0x06 */
#define MORSE_CODE \
{ \
  0x06,0x17,0x15,0x0b,0x03,0x1d,0x09,0x1f,0x07,0x18,0x0a,0x1b,0x04, \
  0x05,0x08,0x19,0x12,0x0d,0x0f,0x02,0x0e,0x1e,0x0c,0x16,0x14,0x13, \
  0x30,0x38,0x3c,0x3e,0x3f,0x2f,0x27,0x23,0x21,0x20,0x6a,0x4c,0x47, \
  0x73,0x61,0x5e,0x2d,0x52,0x6d,0x55,0xf6,0x35,0x7a,0x2a,0x37,0x29, \
  0x2e,0xff,0x01 												    \
}

/*-------------------------------------------------------------------*/

/* Function prototypes produced by cproto */
/* callbacks.c */
void Error_Dialog(char *message, gboolean hide);
void Quit_Dialog(char *message);
void Save_Dialog(char *message);
void Bad_Entry_Dialog(char *message);
/* cat.c */
void Open_Tcvr_Serial(void);
void Open_RTS_Serial(void);
void Read_Tcvr_Serial(unsigned char *cmnd, int block_size);
void Read_Rx_Status(char *mode, char *freq, char *rst);
void Write_Tcvr_Command(unsigned char *cmnd);
void Set_Tcvr_Mode(void);
void Close_Tcvr_Serial(void);
void Tune_Tcvr(double x);
int Read_Rx_Freq(void);
void Write_Rx_Freq(int freq);
void RS232_RTS_Set(gboolean set);
/* display.c */
void Display_Waterfall(void);
void Clear_Wfall_Pixbuf(void);
void Update_Magniphase(int level_i, int level_q);
void Set_TxRx_Labels(void);
void Clear_Record_Fields(gboolean all);
void Print_Character(int printchr, GtkTextBuffer *text_buffer,
	GtkWidget *scrolledwindow, GtkTextIter *iter);
/* fft.c */
void Ifft_Init(void);
void Ifft(void);
/* main.c */
int main(int argc, char *argv[]);
gboolean Load_Config(gpointer data);
int Load_Line(char *buff, FILE *pfile, char *messg);
void Usage(void);
void Cleanup(void);
int isFlagSet(int flag);
int isFlagClear(int flag);
void SetFlag(int flag);
void ClearFlag(int flag);
void ToggleFlag(int flag);
/* operation.c */
gboolean Transmit_Keybd(gpointer data);
gboolean Transmit_Macro(gpointer data);
gboolean Receive_Mode(gpointer data);
gboolean Initialize_Transmit(gpointer data);
gboolean Open_Record_File(void);
/* phase.c */
int PSK31_Phase(void);
int Arc_Tan(int y, int x);
int Scalar_Magn(int x, int y);
/* pskcodec.c */
int Varicode_Bit(void);
int Decode_PSK_Character(void);
void Transmit_Preamble(void);
void Transmit_Postamble(void);
void Transmit_Element(int phase_shift);
void Transmit_PSK_Character(int ascii_chr);
int Viterbi_Decoder(int phase_chg);
int Distance(int A, int B);
unsigned char Parity(unsigned char data);
void Init_Viterbi(void);
void Morse_Transmit(char *mesg);
int Make_Cosine_Table(void);
int Make_Sine_Table(void);
/* sound.c */
void Setup_Sound_Card(void);
gboolean Setup_Dsp_Dev(char *dsp_dev, int mode, int *dsp_fd);
int Signal_Sample(void);
void DSP_Write(unsigned char *buffer, int buff_size);
/* utils.c */
gboolean Save_QSO_Record(void);
void Process_RST(GtkEditable *editable);
void Select_Macro(int macro_num);
int Open_File(FILE **fp, char *fpath, const char *mode);
void Close_File(FILE **fp);
gboolean File_Print(FILE **fp, char *string);
gboolean Locate_Line(char **line_idx, char *line, const char *string, FILE *fp);
gboolean Locate_String(char **line_idx, char *line, const char *string);
gboolean Read_QSO_Record(void);

/*-------------------------------------------------------------------*/

#endif /* End of xpsk31.h */

